#include "UltrasonicSensor.h"

#include <Arduino.h>

namespace RoboidControl {
namespace Arduino {

UltrasonicSensor::UltrasonicSensor(Participant* participant,
                                   unsigned char pinTrigger,
                                   unsigned char pinEcho)
    : Thing(participant) {
  this->pinTrigger = pinTrigger;
  this->pinEcho = pinEcho;

  pinMode(pinTrigger, OUTPUT);  // configure the trigger pin to output mode
  pinMode(pinEcho, INPUT);      // configure the echo pin to input mode
}

UltrasonicSensor::UltrasonicSensor(Thing* parent,
                                   unsigned char pinTrigger,
                                   unsigned char pinEcho)
    : UltrasonicSensor(parent->owner, pinTrigger, pinEcho) {
  this->SetParent(parent);
}

float UltrasonicSensor::GetDistance() {
  // Start the ultrasonic 'ping'
  digitalWrite(pinTrigger, LOW);
  delayMicroseconds(5);
  digitalWrite(pinTrigger, HIGH);
  delayMicroseconds(10);
  digitalWrite(pinTrigger, LOW);

  // Measure the duration of the pulse on the echo pin
  float duration_us =
      pulseIn(pinEcho, HIGH, 100000);  // the result is in microseconds

  // Calculate the distance:
  // * Duration should be divided by 2, because the ping goes to the object
  // and back again. The distance to the object is therefore half the duration
  // of the pulse: duration_us /= 2;
  // * Then we need to convert from microseconds to seconds: duration_sec =
  // duration_us / 1000000;
  // * Now we calculate the distance based on the speed of sound (340 m/s):
  // distance = duration_sec * 340;
  // * The result calculation is therefore:
  this->distance = duration_us / 2 / 1000000 * 340;

  // Filter faulty measurements. The sensor can often give values > 30 m which
  // are not correct
  // if (distance > 30)
  //   distance = 0;

  return distance;
}

void UltrasonicSensor::Update(unsigned long currentTimeMs, bool recursive) {
  GetDistance();
  Thing::Update(currentTimeMs, recursive);
}

UltrasonicSensor::TouchSensor::TouchSensor(Thing& parent,
                                           unsigned char pinTrigger,
                                           unsigned char pinEcho)
    : RoboidControl::TouchSensor(&parent),
      ultrasonic(&parent, pinTrigger, pinEcho) {
  this->touchedSomething = false;
}

void UltrasonicSensor::TouchSensor::Update(unsigned long currentTimeMs,
                                           bool recursive) {
  this->ultrasonic.Update(currentTimeMs, recursive);
  this->touchedSomething = (this->ultrasonic.distance > 0 &&
                            this->ultrasonic.distance <= this->touchDistance);
}

}  // namespace Arduino
}  // namespace RoboidControl